////////////////////////////////////////////////////////////////////////
//
// ColorStructureSAppl.h
//
// This software module was originally developed by
//
// Jim Errico, Sharp Laboratories of America, Camas, WA
// Peter van Beek, Sharp Laboratories of America, Camas, WA
// 
//
// in the course of development of the MPEG-7 standard (ISO/IEC 15938). 
// This software module is an implementation of a part of one or more 
// MPEG-7 tools as specified by the MPEG-7 standard (ISO/IEC 15938).
// ISO/IEC gives users of the MPEG-7 standard free license to this 
// software module or modifications thereof for use in hardware or 
// software products claiming conformance to the MPEG-7 standard. 
// Those intending to use this software module in hardware or software 
// products are advised that its use may infringe existing patents. 
// The original developer of this software module and his/her company, 
// the subsequent editors and their companies, and ISO/IEC have no 
// liability for use of this software module or modifications thereof 
// in an application. No license to this software is granted for non 
// MPEG-7 conforming products. 
// Sharp Laboratories of America retains full right to use the software
// module for their own purpose, assign or donate the software module 
// to a third party and to inhibit third parties from using the software 
// module for non MPEG-7 conforming products. 
//
// Copyright (c) 2000
//
// This copyright notice must be included in all copies or derivative 
// works of this software module.
//
////////////////////////////////////////////////////////////////////////


#ifndef __COLOR_STRUCT_HIST_SERVER__
#define __COLOR_STRUCT_HIST_SERVER__


#include "BasicArchitecture/XMInterfaces.h"
#include "BasicArchitecture/XMObjects.h"

namespace XM
{

//=============================================================================
// This class creates the binary representation of the metadata for the 
// dummy type. 
// 
// For example purposes, each version of the dummy type consists of a
// signed long integer. Thus a library of such dtypes will consists of
// a set of files each containing one (textual representation of an) 
// integer. These are read in and stored in binary form. 
class ColorStructureServer: public ServerApplication
{
public:
	// Null constructor
	ColorStructureServer();

	// Destructor
	virtual ~ColorStructureServer();

	// not used to create descriptor data base
	virtual unsigned long Open();

	// This appears to be the main processing task. It takes in
	// a list of files which should be processed and written out to the
	// file supplied by 'bitstream'.
	virtual unsigned long Start(char *ListFile, char *Bitstream);

	// not used to create descriptor data base
	virtual unsigned long Stop();

	// not used to create descriptor data base
	virtual unsigned long Close();

	virtual const UUID& GetObjectID(void);
	virtual const char *GetName(void);
	
	// Access is allowed only by class factories for this object.  This 
	// avoids having to duplicate the ID definition in multiple locations.
	// In the future, we may have to do this.  PLEASE DO NOT USE THESE
	// UNLESS YOU ARE IMPLEMENTING A CLASS FACTORY GENERATING THIS OBJECT
	static const UUID myID;
	static const char * myName;
private:

}; // End class definition

}; // End namespace definition

#endif //__COLOR_STRUCT_HIST_SERVER__
